package sg;

import java.util.Calendar;
import java.util.Date;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

import org.qcontinuum.astro.RiseSetTime;
import org.qcontinuum.compass.Location;
import org.qcontinuum.compass.LocationLoad;

public class CalendarCanvas extends Canvas implements CommandListener {
  private static final long MILLIS_IN_HOUR = 60 * 60 * 1000;
  private static final long MILLIS_IN_DAY = 24 * MILLIS_IN_HOUR;

  private Font[] fontArr = {
      Font.getFont(Font.FACE_MONOSPACE, Font.STYLE_PLAIN, Font.SIZE_SMALL),
      Font.getFont(Font.FACE_MONOSPACE, Font.STYLE_BOLD, Font.SIZE_SMALL),
      Font.getFont(Font.FACE_MONOSPACE, Font.STYLE_PLAIN, Font.SIZE_MEDIUM),
      Font.getFont(Font.FACE_MONOSPACE, Font.STYLE_BOLD, Font.SIZE_MEDIUM),
      Font.getFont(Font.FACE_MONOSPACE, Font.STYLE_PLAIN, Font.SIZE_LARGE),
      Font.getFont(Font.FACE_MONOSPACE, Font.STYLE_BOLD, Font.SIZE_LARGE)
  };
  private int fontIdx = 2;
  
  private SgTimer sgTimer;
  private Location locationCurrent = new Location();
  private LocationForm locationForm;

  private Calendar calendarFirstRow;
  
  private Command backCommand = new Command("Back (0)", Command.CANCEL, 0);
  private Command locationAddCommand = new Command("Add location", Command.OK, 0);
  private Command locationSelectCommand = new Command("Select location", Command.OK, 0);
  private Command locationEditCommand = new Command("Edit location", Command.OK, 0);
  private Command locationDeleteCommand = new Command("Delete location", Command.OK, 0);
  private Command nextFontCommand = new Command("Change font (#)", Command.OK, 0);
  
  private int linesPerScreen;
  
  public CalendarCanvas(final SgTimer sgTimer) {
    this.sgTimer = sgTimer;
    locationForm = new LocationForm(sgTimer, this, locationCurrent);

    addCommand(backCommand);
    addCommand(locationAddCommand);
    addCommand(locationSelectCommand);
    addCommand(locationEditCommand);
    addCommand(locationDeleteCommand);
    addCommand(nextFontCommand);

    setCommandListener(this);

    calendarFirstRow = Calendar.getInstance();
  }

  protected void paint(final Graphics graphics) {
    int screenWidth = this.getWidth();
    int screenHeight = this.getHeight();

    graphics.setColor(255, 255, 255);
    graphics.fillRect(0, 0, screenWidth, screenHeight);

    Font f = fontArr[fontIdx];
    graphics.setFont(f);
    
    graphics.setColor(0, 0, 0);

    graphics.drawString(locationCurrent.toString(), 0, 0, Graphics.TOP | Graphics.LEFT);
    
    Calendar calendar = Calendar.getInstance(calendarFirstRow.getTimeZone());
    calendar.setTime(calendarFirstRow.getTime());
    
    int fontHeight = f.getHeight();
    
    boolean showYear = true;
    boolean showMonth = true;
    
    int lineWidth = f.stringWidth("2010-01-01 12:12 12:12");
    if(lineWidth > screenWidth) {
      showYear = false;
      lineWidth = f.stringWidth("01-01 12:12 12:12");
      if(lineWidth > screenWidth) {
        showMonth = false;
      }
    }
    
    int lineCount = screenHeight / fontHeight;
    linesPerScreen = lineCount - 1;
    for (int i = 1; i < lineCount; i++) {
      int vertPos = i * fontHeight;
      RiseSetTime riseSetTime =
        new RiseSetTime(
            locationCurrent.getGmt(calendar.getTime()),
            locationCurrent.getDstOffset(calendar.getTime()),
            locationCurrent.getEarthPosition());

      StringBuffer sbRow = new StringBuffer();
      sbRow.append(getDateStr(calendar, showYear, showMonth)).append(" ");
      sbRow.append(riseSetTime.getRiseTime().get24HourTimeString()).append(" ");
      sbRow.append(riseSetTime.getSetTime().get24HourTimeString()).append(" ");
      sbRow.append("(").append(riseSetTime.getTransitTime().get24HourTimeString()).append(")");
      
      graphics.drawString(sbRow.toString(), 0, vertPos, Graphics.TOP | Graphics.LEFT);
   
      calendar = addDay(calendar, 1);
    }
  }
  
    
  private Calendar addDay(final Calendar calendar, final int days) {
    Date date = calendar.getTime();
    long time = date.getTime();
    time += MILLIS_IN_DAY * days;
    date.setTime(time);
    calendar.setTime(date);
    
    return calendar;
  }

  private String getDateStr(final Calendar calendar, final boolean showYear, final boolean showMonth) {
    int day = calendar.get(Calendar.DAY_OF_MONTH);
    
    StringBuffer sbDate = new StringBuffer();
    
    if(showYear) {
      int year = calendar.get(Calendar.YEAR);
      sbDate.append(year).append('-');
    }
    
    if(showMonth) {
      int month = calendar.get(Calendar.MONTH) + 1;
      if(month < 10) {
        sbDate.append('0');
      }
      sbDate.append(month).append('-');
    }
    
    if(day < 10) {
      sbDate.append('0');
    }
    sbDate.append(day);
    
    return sbDate.toString();
  }

  public void commandAction(final Command command, final Displayable displayable) {
    if(command == locationAddCommand) {
      locationCurrent.Name = "";
      locationForm.init();
      SgTimer.show(locationForm);
    }
    else if(command == locationSelectCommand || command == locationDeleteCommand) {
      LocationLoad locationLoad = new LocationLoad(this);
      SgTimer.show(locationLoad);
    }
    else if(command == locationEditCommand) {
      locationForm.init();
      SgTimer.show(locationForm);
    }
    else if(command == backCommand) {
      sgTimer.showTimer();
    }
    else if(command == nextFontCommand) {
      nextFont();
    }
  }
  
  protected void keyPressed(final int keyCode) {
    int gameAction = SgActionUtil.getSgGameAction(this, keyCode);

    switch(gameAction) {
      case Canvas.KEY_NUM0:
        sgTimer.showTimer();
        break;
      case Canvas.KEY_POUND:
        nextFont();
        break;
      case Canvas.UP:
        calendarFirstRow = addDay(calendarFirstRow, -1);
        break;
      case Canvas.DOWN:
        calendarFirstRow = addDay(calendarFirstRow, 1);
        break;
      case Canvas.LEFT:
        calendarFirstRow = addDay(calendarFirstRow, -linesPerScreen);
        break;
      case Canvas.RIGHT:
        calendarFirstRow = addDay(calendarFirstRow, linesPerScreen);
        break;
    }
    
    repaint();
  }
  
  private void nextFont() {
    fontIdx++;
    if(fontIdx >= fontArr.length) {
      fontIdx = 0;
    }
  }

  public void updateLocation(final Location location) {
    this.locationCurrent.updateLocation(location);
  }
  
  public Location getLocationCurrent() {
    return locationCurrent;
  }

  public int getFontIdx() {
    return fontIdx;
  }

  public void setFontIdx(final int fontIdx) {
    this.fontIdx = fontIdx;
  }
}
