package sg;

import java.io.IOException;
import java.util.Timer;
import java.util.TimerTask;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.media.Manager;
import javax.microedition.media.MediaException;
import javax.microedition.media.Player;
import javax.microedition.media.PlayerListener;
import javax.microedition.media.control.ToneControl;
import javax.microedition.media.control.VolumeControl;

public class TimerCanvas extends Canvas implements CommandListener {
  private static final int VIBRATE_LONG = 500;
  private static final int VIBRATE_SHORT = 100;
  private static final int VIBRATE_PAUSE = 500;
  private static final byte STOPPED = 0;
  private static final byte RUNNING = 1;
  private static final byte PAUSED = 2;
  private static final byte ALARM = 3;

  private byte state = STOPPED;

  private byte[] time = { 0, 0, 0, 0, 0, 0 };
  private byte[] originalTime = { 0, 0, 0, 0, 0, 0 };
  private byte[] timeMax = { 9, 9, 5, 9, 5, 9 };

  private byte currentPos = 4;

  private SgTimer sgTimer;

  private Player player;
  private int alarmVolume = 100;
  
  private Command startCommand = new Command("Start/Stop", Command.SCREEN, 1);
  private Command resetCommand = new Command("Reset timer", Command.SCREEN, 2);
  private Command exitCommand = new Command("Exit", Command.SCREEN, 3);
  
  public TimerCanvas(SgTimer sgTimer) {
    this.sgTimer = sgTimer;

    addCommand(startCommand);
    addCommand(resetCommand);
    addCommand(exitCommand);

    setCommandListener(this);
  }

  private class MyTimerTask extends TimerTask {
    public void run() {
      state = RUNNING;
      changeDigit(false, 5);
      if ("00:00:00".equals(getCurrentTimeString())) {
        timerFinished(true);
      }
      else {
        repaint();
      }
    }

    public boolean cancel() {
      return super.cancel();
    }

    public byte getState() {
      return state;
    }
  }

  private MyTimerTask timerTask;

  public void commandAction(Command command, Displayable displayable) {
    if (state == ALARM) {
      stopAlarm();
    }
    else {
      if(command == startCommand) {
        timerStart();
      }
      else if(command == resetCommand) {
        timerFinished(false);
      }
      else if(command == exitCommand) {
        sgTimer.quitApp();
      }
    }
  }

  protected void keyPressed(int keyCode) {
    int gameAction = SgActionUtil.getSgGameAction(this, keyCode);
    
    if (state == ALARM) {
      // Avoid stopping alarm when volume up/down key is pressed (on SE K550i)
      if(gameAction != 0 || keyCode >= 0) {
        // With keys 3 & 9 we change alarm volume, we want to test it while alarm is running
        switch (gameAction) {
          case Canvas.KEY_NUM3:
            updateAlarmVolume(+4);
            break;
          case Canvas.KEY_NUM9:
            updateAlarmVolume(-4);
            break;
          default:
            stopAlarm();
        }
      }
    }
    else if (state == RUNNING) {
      switch (gameAction) {
        case Canvas.FIRE:
          timerStart();
          break;

        case Canvas.KEY_STAR:
          currentTimeLeft();
          break;
      }
    }
    else {
      switch (gameAction) {
        // left/4
        case Canvas.LEFT:
          changeCurrentPos(false);
          break;
        // right/6
        case Canvas.RIGHT:
          changeCurrentPos(true);
          break;
        // up/2
        case Canvas.UP:
          changeCurrentDigit(true);
          break;
        // down/8
        case Canvas.DOWN:
          changeCurrentDigit(false);
          break;

        case Canvas.KEY_NUM0:
          sgTimer.showCalendar();
          break;
          
        case Canvas.FIRE:
          timerStart();
          break;

        case Canvas.KEY_NUM3:
          updateAlarmVolume(+4);
          break;
        case Canvas.KEY_NUM9:
          updateAlarmVolume(-4);
          break;
      }
    }

    repaint();
  }

  private void updateAlarmVolume(int alarmVolumeChange) {
    alarmVolume += alarmVolumeChange;
    if(alarmVolume > 100) {
      alarmVolume = 100;
    }
    else if(alarmVolume < 0) {
      alarmVolume = 0;
    }

    if(player != null) {
      VolumeControl vc = (VolumeControl) player.getControl("VolumeControl");
      vc.setLevel(alarmVolume);
    }
  }

  private void currentTimeLeft() {
    int minTensDigit = time[2];
    int minDigits = time[3];
    
    vibrateLoop(minTensDigit, VIBRATE_LONG);
    vibrateLoop(minDigits, VIBRATE_SHORT);
  }

  private void vibrateLoop(final int timesToVibrate, final int vibrationDuration) {
    for (int i = 0; i < timesToVibrate; i++) {
      Display.getDisplay(sgTimer).vibrate(vibrationDuration);
      try {
        Thread.sleep(vibrationDuration + VIBRATE_PAUSE);
      }
      catch (InterruptedException e) {
        e.printStackTrace();
      }
    }
  }

  private void timerStart() {
    if (timerTask == null) {
      // save set timer time, restore it after timer is stopped or finished
      for (int i = 0; i < time.length; i++) {
        originalTime[i] = time[i];
      }

      timerContinue();
    }
    else {
      switch (timerTask.getState()) {
        case RUNNING:
          timerPause();
          break;
        case PAUSED:
        case STOPPED:
          timerContinue();
          break;
      }
    }
  }

  private void createPlayer() throws Exception {
    player = Manager.createPlayer(this.getClass().getResourceAsStream(
        "/alarm.mid"), "audio/midi");
    player.realize();
    player.addPlayerListener(new PlayerListener() {
      public void playerUpdate(Player player, String event, Object eventData) {
        if (event == PlayerListener.STOPPED
            || event == PlayerListener.END_OF_MEDIA
            || event == PlayerListener.CLOSED) {
          TimerCanvas.this.state = TimerCanvas.STOPPED;
          repaint();
        }
      }
    });
    VolumeControl vc = (VolumeControl) player.getControl("VolumeControl");
    vc.setLevel(alarmVolume);
  }
  
  private void playAlarm() {
    stopAlarm();

    state = ALARM;
    Display.getDisplay(sgTimer).vibrate(2000);
    try {
      createPlayer();
      player.start();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void stopAlarm() {
    state = STOPPED;

    if (player != null) {
      try {
        player.stop();
      }
      catch (MediaException e) {
        e.printStackTrace();
      }
      player.deallocate();
    }

    player = null;
    System.gc();

    repaint();
  }

  private void timerFinished(boolean startPlayingAlarm) {
    this.state = STOPPED;

    if (timerTask != null) {
      timerTask.cancel();
      timerTask = null;
      System.gc();
      if (startPlayingAlarm) {
        playAlarm();
      }

      // restore original timer time after timer is stopped or finished
      for (int i = 0; i < time.length; i++) {
        time[i] = originalTime[i];
      }

      repaint();
    }
  }

  public void timerQuit() {
    state = STOPPED;
    if (timerTask != null) {
      timerTask.cancel();
      timerTask = null;
      System.gc();
    }
  }

  private void timerPause() {
    Display.getDisplay(sgTimer).vibrate(VIBRATE_LONG);
    state = PAUSED;
    timerTask.cancel();
  }

  private void timerContinue() {
    Display.getDisplay(sgTimer).vibrate(VIBRATE_SHORT);
    timerTask = null;
    System.gc();
    timerTask = new MyTimerTask();

    Timer timer = new Timer();
    timer.scheduleAtFixedRate(timerTask, 0, 1000);
  }

  private void changeCurrentPos(boolean add) {
    if (add) {
      currentPos++;
      switch (currentPos) {
        case 6:
          currentPos = 0;
          break;
      }
    }
    else {
      currentPos--;
      switch (currentPos) {
        case -1:
          currentPos = 5;
          break;
      }
    }
  }

  private void changeCurrentDigit(boolean add) {
    if (timerTask == null) {
      changeDigit(add, currentPos);
    }
  }

  private void changeDigit(boolean add, int digitPos) {
    byte currentDigit = time[digitPos];
    if (add) {
      currentDigit = (byte) (currentDigit + 1);
    }
    else {
      currentDigit = (byte) (currentDigit - 1);
    }

    byte digitMax = timeMax[digitPos];

    if (currentDigit > digitMax) {
      currentDigit = 0;
      if (digitPos > 0) {
        changeDigit(true, digitPos - 1);
      }
    }
    else if (currentDigit < 0) {
      currentDigit = digitMax;
      if (digitPos > 0) {
        changeDigit(false, digitPos - 1);
      }
    }

    time[digitPos] = currentDigit;
  }

  protected void keyReleased(int keyCode) {
  }

  protected void keyRepeated(int keyCode) {
  }

  private String getCurrentTimeString() {
    return "" + time[0] + time[1] + ":" + time[2] + time[3] + ":" + time[4]
        + time[5];
  }

  private String getCurrentDigitString() {
    int currentIdx = getCurrentIdx();
    return getCurrentTimeString().substring(currentIdx, currentIdx + 1);
  }

  private int getCurrentIdx() {
    // 01 23 45
    // 01234567
    // 00:00:00
    switch (currentPos) {
      case 2:
      case 3:
        return currentPos + 1;
      case 4:
      case 5:
        return currentPos + 2;
      default:
        return currentPos;
    }
  }

  protected void paint(Graphics graphics) {
    int screenWidth = this.getWidth();
    int screenHeight = this.getHeight();

    graphics.setColor(255, 255, 255);
    graphics.fillRect(0, 0, screenWidth, screenHeight);

    Font f = Font.getFont(Font.FACE_MONOSPACE, Font.STYLE_BOLD, Font.SIZE_LARGE);
    graphics.setFont(f);

    String timeString = getCurrentTimeString();

    String statusString = getCurrentStatusString();

    paintTime(graphics, screenHeight / 2, timeString);
    paintCurrentDigit(graphics, screenHeight / 2, timeString);
    paintStatus(graphics, screenHeight / 2 + f.getHeight(), statusString);
    paintAlarmVolume(graphics, screenWidth, screenHeight);
    
    paintCalendarMessage(graphics);
  }

  private void paintAlarmVolume(Graphics graphics, int screenWidth, int screenHeight) {
    Font f = Font.getFont(Font.FACE_MONOSPACE, Font.STYLE_PLAIN, Font.SIZE_MEDIUM);
    graphics.setFont(f);
    graphics.setColor(0, 0, 0);

    int textHeight = f.getHeight();
    int rectWidth = f.stringWidth("9") - 1;
    int rectTop = textHeight;
    int rectHeight = screenHeight - 3 * textHeight - 1;
    int fillHeight = rectHeight * alarmVolume / 100;
    int fillTop = rectTop + rectHeight - fillHeight;
    graphics.drawString("volume up: 3", screenWidth - 1, 0, Graphics.TOP | Graphics.RIGHT);
    graphics.drawString("volume down: 9", screenWidth - 1, screenHeight - textHeight, Graphics.BOTTOM | Graphics.RIGHT);
    graphics.drawRect(screenWidth - rectWidth - 2, rectTop, rectWidth, rectHeight);
    graphics.fillRect(screenWidth - rectWidth - 2, fillTop, rectWidth, fillHeight);
    
  }

  private String getCurrentStatusString() {
    switch (state) {
      case STOPPED:
        return "STOPPED";
      case PAUSED:
        return "PAUSED";
      case RUNNING:
        return "RUNNING";
      case ALARM:
        return "ALARM";
      default:
        return "?";
    }
  }

  private void paintTime(Graphics graphics,  int vertPos, String timeString) {
    Font f = graphics.getFont();
    int timeStringWidth = f.stringWidth(timeString);
    int timeStringPos = (getWidth() - timeStringWidth) / 2;
    graphics.setColor(0, 0, 0);
    graphics.drawString(timeString, timeStringPos, vertPos, Graphics.BOTTOM | Graphics.LEFT);
  }

  private void paintCurrentDigit(Graphics graphics, int vertPos, String timeString) {
    Font f = graphics.getFont();
    int timeStringWidth = f.stringWidth(timeString);
    int timeStringPos = (getWidth() - timeStringWidth) / 2;

    graphics.setColor(255, 0, 0);
    // 01234567
    String digitStr = getCurrentDigitString();
    int currentIdx = getCurrentIdx();
    int digitPos = timeStringPos + f.stringWidth(timeString.substring(0, currentIdx));

    graphics.drawString(digitStr, digitPos, vertPos, Graphics.BOTTOM | Graphics.LEFT);
  }

  private void paintStatus(Graphics graphics, int vertPos, String statusString) {
    Font f = graphics.getFont();
    int statusStringWidth = f.stringWidth(statusString);
    int statusStringPos = (getWidth() - statusStringWidth) / 2;
    graphics.setColor(0, 0, 0);
    graphics.drawString(statusString, statusStringPos, vertPos, Graphics.BOTTOM | Graphics.LEFT);
  }

  private void paintCalendarMessage(Graphics graphics) {
    String msg = "Press '0' to show calendar";

    Font f = Font.getFont(Font.FACE_MONOSPACE, Font.STYLE_PLAIN, Font.SIZE_MEDIUM);
    int strWidth = f.stringWidth(msg);
    int calendarMsgPos = (getWidth() - strWidth) / 2;
    graphics.setFont(f);
    graphics.setColor(0, 0, 0);
    graphics.drawString(msg, calendarMsgPos, getHeight(), Graphics.BOTTOM | Graphics.LEFT);
  }
  
  public byte[] getTime() {
    return time;
  }

  public void setTime(byte[] time) {
    this.time = time;
  }

  public byte[] getOriginalTime() {
    return originalTime;
  }

  public void setOriginalTime(byte[] originalTime) {
    this.originalTime = originalTime;
  }

  public boolean isRunning() {
    return this.timerTask != null;
  }

  public int getAlarmVolume() {
    return alarmVolume;
  }

  public void setAlarmVolume(int alarmVolume) {
    this.alarmVolume = alarmVolume;
  }

  public void setRunning(boolean running) {
    if (running) {
      this.timerTask = new MyTimerTask();
      this.state = PAUSED;
    }
    else {
      this.timerTask = null;
      System.gc();
    }
  }
}
